import 'package:flutter/foundation.dart';

enum BuildType {
  DEBUG,
  PROFILE,
  RELEASE,
}

class BuildConfig<V> {
  final BuildType buildType = appBuildType;
  final String packageName;
  final String appName;
  final String versionCode;
  final String versionName;
  final V variant;

  BuildConfig({
    required this.packageName,
    required this.appName,
    required this.versionCode,
    required this.versionName,
    required this.variant,
  });

  static BuildType get appBuildType {
    if (kReleaseMode) return BuildType.RELEASE;
    if (kProfileMode) return BuildType.PROFILE;
    return BuildType.DEBUG;
  }

  bool get isDebug => buildType == BuildType.DEBUG;
  bool get isProfile => buildType == BuildType.PROFILE;
  bool get isRelease => buildType == BuildType.RELEASE;

  @override
  String toString() => 'buildType: $buildType,\n'
      'packageName: $packageName,\n'
      'appName: $appName,\n'
      'versionCode: $versionCode,\n'
      'versionName: $versionName,\n'
      'variant: $variant,\n';
}
