import 'package:adeptutils/src/collections/multi_key_map.dart';

class MapBuilder<K, V> {
  late Map<K, V> _map;
  bool multiKey;

  MapBuilder({this.multiKey = false}) {
    clear();
  }

  MapBuilder<K, V> put(K key, V value) {
    _map[key] = value;
    return this;
  }

  MapBuilder<K, V> putAll(Map<K, V>? map) {
    if (map != null) _map.addAll(map);
    return this;
  }

  MapBuilder clear() {
    _map = multiKey ? MultiKeyMap<K, V>() : {};
    return this;
  }

  bool remove(K key, {V? value}) {
    if (key != null && value != null) {
      V? currentValue = get(key);
      if (currentValue == value) {
        return _map.remove(key) != null;
      }
    } else if (key != null && value == null) {
      return _map.remove(key) != null;
    }

    return false;
  }

  V? removeKey(K key) => _map.remove(key);

  bool containsKey(K key) => _map.containsKey(key);
  bool containsValue(V value) => _map.containsValue(value);

  V? get(K key) => _map[key];

  bool get isEmpty => _map.isEmpty;
  bool get isNotEmpty => _map.isNotEmpty;

  Map<K, V> build() => _map;

  static Map empty({bool multiKey = false}) {
    return multiKey ? MultiKeyMap() : {};
  }
}
