import 'package:adeptutils/src/extendables/jsonable.dart';
import 'package:adeptutils/src/collections/multi_entry_set.dart';
import 'package:adeptutils/src/collections/jmap_entry.dart';

class MultiKeyMapUtils {
  static Map<String, dynamic> toJson(MultiKeyMap<String, dynamic> value) =>
      value.toJson();

  static MultiKeyMap<String, dynamic>? fromJson(Map<String, dynamic>? value) =>
      value == null ? null : MultiKeyMap<String, dynamic>.fromJson(value);
}

class MultiKeyMap<K, V> extends Jsonable<MultiKeyMap> implements Map<K, V> {
  late List<JMapEntry<K, V>> items;

  MultiKeyMap() {
    items = [];
  }

  @override
  bool get isNotEmpty => items.isNotEmpty;

  @override
  bool get isEmpty => items.isEmpty;

  @override
  int get length => items.length;

  @override
  Iterable<V> get values {
    List<V> values = [];
    for (JMapEntry<K, V> entry in items) {
      values.add(entry.value);
    }
    return values;
  }

  @override
  Iterable<K> get keys {
    Set<K> keySet = MultiEntrySet<K>();
    for (JMapEntry<K, V> entry in items) {
      keySet.add(entry.key);
    }
    return keySet;
  }

  @override
  void forEach(void Function(K key, V value) f) {
    for (var entry in items) {
      f(entry.key, entry.value);
    }
  }

  @override
  void clear() {
    items = [];
  }

  @override
  V? remove(Object? key) {
    int i = 0;
    for (JMapEntry<K, V> entry in items) {
      if (entry.key == key) {
        V value = entry.value;
        items.removeAt(i);
        return value;
      }
      ++i;
    }
    return null;
  }

  @override
  void addAll(Map<K, V> other) {
    for (MapEntry entry in other.entries) {
      items.add(JMapEntry(entry.key, entry.value));
    }
  }

  @override
  V putIfAbsent(K key, V Function() ifAbsent) {
    for (var entry in items) {
      print(entry);
    }
    return ifAbsent.call();
  }

  @override
  void removeWhere(bool Function(K key, V value) predicate) {
    for (var entry in items) {
      print(entry);
    }
  }

  @override
  void updateAll(V Function(K key, V value) update) {
    for (var entry in items) {
      print(entry);
    }
  }

  @override
  V update(K key, V Function(V value) update, {V Function()? ifAbsent}) {
    for (var entry in items) {
      print(entry);
    }
    return ifAbsent?.call() as V;
  }

  @override
  void addEntries(Iterable<MapEntry<K, V>> newEntries) {
    for (var entry in newEntries) {
      items.add(JMapEntry(entry.key, entry.value));
    }
  }

  @override
  Iterable<MapEntry<K, V>> get entries {
    List<MapEntry<K, V>> list = [];
    for (var entry in items) {
      list.add(MapEntry(entry.key, entry.value));
    }
    return list;
  }

  @override
  void operator []=(K key, V value) => items.add(JMapEntry<K, V>(key, value));

  @override
  V? operator [](Object? key) {
    for (JMapEntry<K, V> entry in items) {
      if (entry.key == key) return entry.value;
    }
    return null;
  }

  @override
  bool containsKey(Object? key) {
    for (JMapEntry entry in items) {
      if (entry.key == key) return true;
    }
    return false;
  }

  @override
  bool containsValue(Object? value) {
    for (JMapEntry entry in items) {
      if (entry.value == value) return true;
    }
    return false;
  }

  @override
  Map<K2, V2> map<K2, V2>(MapEntry<K2, V2> Function(K key, V value) f) {
    return {};
  }

  @override
  Map<RK, RV> cast<RK, RV>() {
    return {};
  }

  List<JMapEntry<K, V>> toList() => items;

  @override
  Map<String, dynamic> toJson() =>
      <String, dynamic>{'multi_key_map_items': items};

  @override
  MultiKeyMap<String, dynamic>? fromJsonMap(Map<String, dynamic>? json) =>
      json == null ? null : MultiKeyMap.fromJson(json);

  factory MultiKeyMap.fromJson(Map<String, dynamic> json) {
    if (json['multi_key_map_items'] != null) {
      return MultiKeyMap<K, V>()
        ..items = (json['multi_key_map_items']
                .map((e) => JMapEntry<K, V>.fromJson(e as Map<String, dynamic>))
                .toList()) ??
            [];
    }
    List<JMapEntry<K, V>> items = [];
    json.forEach((key, value) {
      items.add(JMapEntry<K, V>.fromJson({'key': key, 'value': value}));
    });
    return MultiKeyMap()..items = items;
  }
}
