import 'dart:collection';
import 'package:adeptutils/src/extendables/jsonable.dart';
//import 'package:json_annotation/json_annotation.dart';

//part 'queue_set.g.dart';
//
//@JsonSerializable()
class QueueSet<T> extends Jsonable<QueueSet<T>> {
  late int size;
  late ListQueue<T> queue;

  QueueSet({
    ListQueue<T>? queue,
    int? size,
  }) {
    this.queue = queue ?? ListQueue();
    this.size = size ?? this.queue.length;
  }

  bool contains(Object? o) => queue.contains(o);

  T poll() {
    T t = queue.removeFirst();
    --size;
    return t;
  }

  T get peek => queue.first;

  void add(T t) {
    if (!contains(t)) {
      queue.add(t);
      ++size;
    }
  }

  bool get isEmpty => queue.isEmpty;

  void addAll(Iterable<T> c) {
    if (c.isEmpty) return;
    queue.addAll(c);
    size += c.length;
  }

  void clear() {
    queue = ListQueue();
    size = 0;
  }

  @override
  Map<String, dynamic> toJson() => <String, dynamic>{
        'size': size,
        'queue': queue as List?,
      };

  @override
  QueueSet<T>? fromJsonMap(Map<String, dynamic>? json) =>
      json == null ? null : QueueSet.fromJson(json);

  factory QueueSet.fromJson(Map<String, dynamic> json) =>
      QueueSet(size: json['size'], queue: json['queue'] as ListQueue<T>?);
}
