import 'package:date_format/date_format.dart' as DateFormatter;

class DateUtils {
  static const List<String> DATE_FORMAT = [
    'yyyy',
    '-',
    'mm',
    '-',
    'dd',
    'T',
    'HH',
    ':',
    'nn',
    ':',
    'ss'
  ];

  static String? convertToString(
      {DateTime? dateTime, int? timeInMillis, List<String>? dateFormat}) {
    if (timeInMillis != null) {
      return convertToString(
          dateTime: DateTime.fromMillisecondsSinceEpoch(timeInMillis),
          dateFormat: dateFormat);
    }
    return dateTime != null
        ? format(dateTime: dateTime, dateFormat: dateFormat)
        : null;
  }

  static DateTime? convertToDateTime({String? dateString, int? timeInMillis}) {
    if (dateString != null) return DateTime.tryParse(dateString);
    return timeInMillis != null
        ? DateTime.fromMillisecondsSinceEpoch(timeInMillis)
        : null;
  }

  static int? convertToTimeInMillis({DateTime? dateTime, String? dateString}) {
    if (dateString != null) {
      return convertToTimeInMillis(dateTime: DateTime.tryParse(dateString));
    }
    return dateTime?.millisecondsSinceEpoch;
  }

  static String? format({DateTime? dateTime, List<String>? dateFormat}) {
    return dateTime != null
        ? DateFormatter.formatDate(dateTime, dateFormat ?? DATE_FORMAT)
        : null;
  }

  static int nowUntilHour() => nowUntilHourAsDateTime().millisecondsSinceEpoch;
  static DateTime nowUntilHourAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day, now.hour);
  }

  static int nowUntilMinutes() =>
      nowUntilMinutesAsDateTime().millisecondsSinceEpoch;
  static DateTime nowUntilMinutesAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day, now.hour, now.minute);
  }

  static int nowUntilSeconds() =>
      nowUntilSecondsAsDateTime().millisecondsSinceEpoch;
  static DateTime nowUntilSecondsAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(
        now.year, now.month, now.day, now.hour, now.minute, now.second);
  }

  static int now() => nowAsDateTime().millisecondsSinceEpoch;
  static DateTime nowAsDateTime() => DateTime.now();

  static int yesterday() => yesterdayAsDateTime().millisecondsSinceEpoch;
  static DateTime yesterdayAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day - 1);
  }

  static int today() => todayAsDateTime().millisecondsSinceEpoch;
  static DateTime todayAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day);
  }

  static int tomorrow() => tomorrowAsDateTime().millisecondsSinceEpoch;
  static DateTime tomorrowAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day + 1);
  }

  static int tomorrowSameTime() =>
      tomorrowSameTimeAsDateTime().millisecondsSinceEpoch;
  static DateTime tomorrowSameTimeAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day + 1, now.hour, now.minute,
        now.second, now.millisecond);
  }

  static int dayAfterTomorrow() =>
      dayAfterTomorrowAsDateTime().millisecondsSinceEpoch;
  static DateTime dayAfterTomorrowAsDateTime() {
    DateTime now = DateTime.now();
    return DateTime(now.year, now.month, now.day + 2);
  }

  static bool isWithin24Hours(
      {DateTime? dateTime, int? millisecondsSinceEpoch}) {
    if (dateTime == null && millisecondsSinceEpoch == null) return false;
    if (dateTime != null) {
      return isWithin24Hours(
          millisecondsSinceEpoch: dateTime.millisecondsSinceEpoch);
    }
    return (DateTime.now().millisecondsSinceEpoch - millisecondsSinceEpoch!) <=
        const Duration(hours: 24).inMilliseconds;
  }
}
