import 'package:flutter/material.dart';
import 'package:adeptutils/src/text_utils.dart';
import 'dart:io';

class GenericException implements Exception {
  String? _message;
  Exception? _causeException;
  Error? _causeError;
  GenericException(
      {String? message, Exception? causeException, Error? causeError}) {
    if (TextUtils.isEmpty(message)) {
      try {
        if (causeException != null) {
          int indexOfColon = causeException.toString().indexOf(':');
          if (indexOfColon != -1) {
            message =
                causeException.toString().substring(indexOfColon + 1).trim();
          }
        }
      } catch (e) {
        print(e);
      }
    }
    if (TextUtils.isEmpty(message)) {
      if (causeError != null) message = causeError.toString();
    }
    if (TextUtils.isEmpty(message)) message = '';
    _causeException = causeException;
    _causeError = causeError;
    _message = message;
  }
  String? get message => _message;
  Exception? get causeException => _causeException;
  Error? get causeError => _causeError;
  @override
  String toString() => message!;
}

class ExceptionUtils {
  static GenericException parseException(dynamic e) {
    try {
      if (e is Exception) {
        return GenericException(causeException: e);
      } else if (e is Error) {
        return GenericException(causeError: e);
      } else {
        return GenericException(message: 'Unknown exception');
      }
    } catch (e) {
      return GenericException(message: 'Unknown exception');
    }
  }

  static void handleException(dynamic e, BuildContext context,
      {String title = 'Error',
      String btnContinueText = 'CONTINUE',
      String btnExitText = 'EXIT'}) {
    Exception ex = parseException(e);
    showDialog(
      context: context,
      barrierDismissible: false,
      builder: (context) => AlertDialog(
        title: Text(title),
        content: Text(ex.toString()),
        actions: <Widget>[
          TextButton(
            child: Text(btnContinueText),
            onPressed: () {
              Navigator.of(context).pop();
            },
          ),
          TextButton(
            child: Text(btnExitText),
            onPressed: () {
              Navigator.of(context).pop();
              if (Navigator.of(context).canPop()) {
                Navigator.of(context).pop();
              } else {
                exit(0);
              }
            },
          )
        ],
      ),
    );
  }
}
