import 'package:url_launcher/url_launcher.dart';

class IntentUtils {
  static Future<bool> launchURL(String url,
      {bool ensureURLScheme = false,
      bool forceExternalApplication = false}) async {
    try {
      final String suitableURL;
      if (ensureURLScheme &&
          !url.startsWith('http://') &&
          !url.startsWith('https://') &&
          !url.startsWith('ftp://') &&
          !url.startsWith('file://')) {
        suitableURL = 'http://$url';
      } else {
        suitableURL = url;
      }

      final uri = Uri.parse(suitableURL);

      if (forceExternalApplication) {
        bool externalAppLaunchSucceeded =
            await launchUrl(uri, mode: LaunchMode.externalApplication);
        if (externalAppLaunchSucceeded) return true;
      }

      return await launchUrl(uri);
    } catch (e) {
      print(e);
    }
    return false;
  }

  static Future<bool> launchWhatsApp(String phoneNumber) async =>
      await launchURL('https://wa.me/$phoneNumber');

  static Future<bool> launchMessenger(String username) async =>
      await launchURL('https://m.me/$username');

  static Future<bool> launchTelegram(String username) async =>
      await launchURL('https://telegram.me/$username');

  static Future<bool> sendEmail(String address,
      {String subject = '', String body = ''}) async {
    try {
      return launchUrl(
          Uri.parse('mailto:$address?subject=$subject&body=$body'));
    } catch (e) {
      print(e);
    }
    return false;
  }

  static Future<bool> sendSMS(String phoneNumber) async {
    try {
      return launchUrl(Uri.parse('sms:${phoneNumber.split(' ').join()}'));
    } catch (e) {
      print(e);
    }
    return false;
  }

  static Future<bool> call(String phoneNumber) async {
    try {
      return launchUrl(Uri.parse('tel:${phoneNumber.split(' ').join()}'));
    } catch (e) {
      print(e);
    }
    return false;
  }
}
