import 'dart:io';
import 'package:flutter/foundation.dart';

enum PlatformType {
  Android,
  Fuchsia,
  IOS,
  Linux,
  MacOS,
  Web,
  Windows,
  Unknown,
}

extension PlatformUtils on PlatformType {
  static final PlatformType _type = _init();

  String get name => describeEnum(this);

  static PlatformType get type => _type;

  static bool get isMobile {
    switch (type) {
      case PlatformType.Android:
      case PlatformType.IOS:
        return true;
      default:
        return false;
    }
  }

  static bool get isOS {
    switch (type) {
      case PlatformType.Android:
      case PlatformType.Fuchsia:
      case PlatformType.IOS:
      case PlatformType.Linux:
      case PlatformType.MacOS:
      case PlatformType.Windows:
        return true;
      default:
        return false;
    }
  }

  static bool get isAndroid => type == PlatformType.Android;
  static bool get isFuchsia => type == PlatformType.Fuchsia;
  static bool get isIOS => type == PlatformType.IOS;
  static bool get isLinux => type == PlatformType.Linux;
  static bool get isMacOS => type == PlatformType.MacOS;
  static bool get isWeb => type == PlatformType.Web;
  static bool get isWindows => type == PlatformType.Windows;

  static PlatformType _init() {
    try {
      //Web checking has to be the first one to avoid "This is no an operating system" exception
      if (kIsWeb) return PlatformType.Web;
      if (Platform.isAndroid) return PlatformType.Android;
      if (Platform.isFuchsia) return PlatformType.Fuchsia;
      if (Platform.isIOS) return PlatformType.IOS;
      if (Platform.isLinux) return PlatformType.Linux;
      if (Platform.isMacOS) return PlatformType.MacOS;
      if (Platform.isWindows) return PlatformType.Windows;
      return PlatformType.Unknown;
    } catch (e) {
      print(e);
    }
    return PlatformType.Unknown;
  }
}
