import 'package:adeptutils/adeptutils.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class ScreenFormFactor {
  static const MINI = 100;
  static const MINI_1 = 150;
  static const SMALL = 200;
  static const SMALL_1 = 250;
  static const NORMAL = MOBILE;
  static const NORMAL_1 = 350;
  static const NORMAL_2 = 400;
  static const NORMAL_3 = 450;
  static const NORMAL_4 = 500;
  static const NORMAL_5 = 550;
  static const MEDIUM = TABLET;
  static const MEDIUM_1 = 650;
  static const MEDIUM_2 = 700;
  static const MEDIUM_3 = 750;
  static const MEDIUM_4 = 800;
  static const MEDIUM_5 = 850;
  static const LARGE = DESKTOP;
  static const LARGE_1 = 950;
  static const LARGE_2 = 1000;
  static const LARGE_3 = 1050;
  static const LARGE_4 = 1100;
  static const LARGE_5 = 1150;
  static const X_LARGE = TV;

  static const MOBILE = 300;

  /// sw600dp qualifier is the breakpoint for popular 7-inch Android tablets.
  /// sw means smallest width, and the number 600 here is a common breakpoint
  /// for a typical 7-inch tablet.
  static const TABLET = 600;
  static const DESKTOP = 900;
  static const TV = 1200;
}

enum DeviceScreenType {
  WATCH,
  MOBILE,
  TABLET,
  DESKTOP,
  TV,
}

enum DeviceScreenMeasure {
  TINY,
  MINI,
  MINI_1,
  SMALL,
  SMALL_1,
  NORMAL,
  NORMAL_1,
  NORMAL_2,
  NORMAL_3,
  NORMAL_4,
  NORMAL_5,
  MEDIUM,
  MEDIUM_1,
  MEDIUM_2,
  MEDIUM_3,
  MEDIUM_4,
  MEDIUM_5,
  LARGE,
  LARGE_1,
  LARGE_2,
  LARGE_3,
  LARGE_4,
  LARGE_5,
  X_LARGE,
}

class ScreenUtils {
  @Deprecated(
      'Use [getFlutterViewSize(context: context)] instead as this function will be removed in later versions')
  static Size get windowSize {
    final flutterView = FlutterViewUtils.get();
    //this is the same as MediaQueryData.fromWindow(window).size but faster
    return (flutterView.physicalSize / flutterView.devicePixelRatio);
  }

  //this is the same as MediaQueryData.fromWindow(window).size but faster
  static Size getFlutterViewSize({BuildContext? context}) {
    final flutterView = FlutterViewUtils.get(context: context);
    return (flutterView.physicalSize / flutterView.devicePixelRatio);
  }

  static DeviceScreenMeasure getMeasure({Size? size, BuildContext? context}) {
    final shortestSide =
        (size ?? getFlutterViewSize(context: context)).shortestSide;
    if (shortestSide > ScreenFormFactor.X_LARGE) {
      return DeviceScreenMeasure.X_LARGE;
    }
    if (shortestSide > ScreenFormFactor.LARGE_5) {
      return DeviceScreenMeasure.LARGE_5;
    }
    if (shortestSide > ScreenFormFactor.LARGE_4) {
      return DeviceScreenMeasure.LARGE_4;
    }
    if (shortestSide > ScreenFormFactor.LARGE_3) {
      return DeviceScreenMeasure.LARGE_3;
    }
    if (shortestSide > ScreenFormFactor.LARGE_2) {
      return DeviceScreenMeasure.LARGE_2;
    }
    if (shortestSide > ScreenFormFactor.LARGE_1) {
      return DeviceScreenMeasure.LARGE_1;
    }
    if (shortestSide > ScreenFormFactor.LARGE) return DeviceScreenMeasure.LARGE;
    if (shortestSide > ScreenFormFactor.MEDIUM_5) {
      return DeviceScreenMeasure.MEDIUM_5;
    }
    if (shortestSide > ScreenFormFactor.MEDIUM_4) {
      return DeviceScreenMeasure.MEDIUM_4;
    }
    if (shortestSide > ScreenFormFactor.MEDIUM_3) {
      return DeviceScreenMeasure.MEDIUM_3;
    }
    if (shortestSide > ScreenFormFactor.MEDIUM_2) {
      return DeviceScreenMeasure.MEDIUM_2;
    }
    if (shortestSide > ScreenFormFactor.MEDIUM_1) {
      return DeviceScreenMeasure.MEDIUM_1;
    }
    if (shortestSide > ScreenFormFactor.MEDIUM) {
      return DeviceScreenMeasure.MEDIUM;
    }
    if (shortestSide > ScreenFormFactor.NORMAL_5) {
      return DeviceScreenMeasure.NORMAL_5;
    }
    if (shortestSide > ScreenFormFactor.NORMAL_4) {
      return DeviceScreenMeasure.NORMAL_4;
    }
    if (shortestSide > ScreenFormFactor.NORMAL_3) {
      return DeviceScreenMeasure.NORMAL_3;
    }
    if (shortestSide > ScreenFormFactor.NORMAL_2) {
      return DeviceScreenMeasure.NORMAL_2;
    }
    if (shortestSide > ScreenFormFactor.NORMAL_1) {
      return DeviceScreenMeasure.NORMAL_1;
    }
    if (shortestSide > ScreenFormFactor.NORMAL) {
      return DeviceScreenMeasure.NORMAL;
    }
    if (shortestSide > ScreenFormFactor.SMALL_1) {
      return DeviceScreenMeasure.SMALL_1;
    }
    if (shortestSide > ScreenFormFactor.SMALL) return DeviceScreenMeasure.SMALL;
    if (shortestSide > ScreenFormFactor.MINI_1) {
      return DeviceScreenMeasure.MINI_1;
    }
    if (shortestSide > ScreenFormFactor.MINI) return DeviceScreenMeasure.MINI;
    return DeviceScreenMeasure.TINY;
  }

  static DeviceScreenType getType({Size? size, BuildContext? context}) {
    // Use .shortestSide to detect device type regardless of orientation
    final shortestSide =
        (size ?? getFlutterViewSize(context: context)).shortestSide;
    if (shortestSide > ScreenFormFactor.TV) return DeviceScreenType.TV;
    if (shortestSide > ScreenFormFactor.DESKTOP) {
      return DeviceScreenType.DESKTOP;
    }
    if (shortestSide > ScreenFormFactor.TABLET) return DeviceScreenType.TABLET;
    if (shortestSide > ScreenFormFactor.MOBILE) return DeviceScreenType.MOBILE;
    return DeviceScreenType.WATCH;
  }

  static bool isWatch({Size? size, BuildContext? context}) =>
      getType(size: size, context: context) == DeviceScreenType.WATCH;

  static bool isMobile({Size? size, BuildContext? context}) =>
      getType(size: size, context: context) == DeviceScreenType.MOBILE;

  static bool isTablet({Size? size, BuildContext? context}) =>
      getType(size: size, context: context) == DeviceScreenType.TABLET;

  static bool isDesktop({Size? size, BuildContext? context}) =>
      getType(size: size, context: context) == DeviceScreenType.DESKTOP;

  static bool isTV({Size? size, BuildContext? context}) =>
      getType(size: size, context: context) == DeviceScreenType.TV;

  @Deprecated('Use isMobile instead')
  static bool isMobileScreen({Size? size, BuildContext? context}) =>
      (size ?? windowSize).shortestSide < ScreenFormFactor.TABLET;

  @Deprecated('Use isTablet instead')
  static bool isTabletScreen({Size? size, BuildContext? context}) =>
      !isMobileScreen(size: size, context: context);

  static bool isPortrait({BuildContext? context}) {
    if (context != null) {
      return MediaQuery.of(context).orientation == Orientation.portrait;
    }
    return MediaQueryData.fromView(FlutterViewUtils.get(context: context))
            .orientation ==
        Orientation.portrait;
  }

  static bool isLandscape({BuildContext? context}) {
    if (context != null) {
      return MediaQuery.of(context).orientation == Orientation.landscape;
    }
    return MediaQueryData.fromView(FlutterViewUtils.get(context: context))
            .orientation ==
        Orientation.landscape;
  }

  /// To use this function make sure to call
  /// WidgetsFlutterBinding.ensureInitialized(); in your main() function
  static Future<void> clearPreferredOrientation() =>
      setPreferredOrientations([]);

  /// To use this function make sure to call
  /// WidgetsFlutterBinding.ensureInitialized(); in your main() function
  static Future<void> setPreferredOrientationPortrait() =>
      setPreferredOrientations([DeviceOrientation.portraitUp]);

  /// To use this function make sure to call
  /// WidgetsFlutterBinding.ensureInitialized(); in your main() function
  static Future<void> setPreferredOrientationLandscape() =>
      setPreferredOrientations(
          [DeviceOrientation.landscapeLeft, DeviceOrientation.landscapeRight]);

  /// To use this function make sure to call
  /// WidgetsFlutterBinding.ensureInitialized(); in your main() function
  static Future<void> setPreferredOrientation(DeviceOrientation orientation) =>
      setPreferredOrientations([orientation]);

  /// To use this function make sure to call
  /// WidgetsFlutterBinding.ensureInitialized(); in your main() function
  static Future<void> setPreferredOrientations(
          List<DeviceOrientation> orientations) =>
      SystemChrome.setPreferredOrientations(orientations);
}
