import 'package:shared_preferences/shared_preferences.dart';

class SharedPreferencesUtils {
  static late SharedPreferences sharedPreferences;

  //Singleton Pattern
  static Future<SharedPreferences> init() async {
    sharedPreferences = await SharedPreferences.getInstance();
    return sharedPreferences;
  }

  static Future<bool> putBool(String key, bool? value) =>
      value == null ? remove(key) : sharedPreferences.setBool(key, value);

  static bool? getBool(String key, {bool? defaultValue}) {
    bool? value = sharedPreferences.getBool(key);
    return value ?? defaultValue;
  }

  static Future<bool> putString(String key, String? value) =>
      value == null ? remove(key) : sharedPreferences.setString(key, value);

  static String? getString(String key, {String? defaultValue}) {
    String? value = sharedPreferences.getString(key);
    return value ?? defaultValue;
  }

  static Future<bool> putInt(String key, int? value) =>
      value == null ? remove(key) : sharedPreferences.setInt(key, value);

  static int? getInt(String key, {int? defaultValue}) {
    int? value = sharedPreferences.getInt(key);
    return value ?? defaultValue;
  }

  static Future<bool> putDouble(String key, double? value) =>
      value == null ? remove(key) : sharedPreferences.setDouble(key, value);

  static double? getDouble(String key, {double? defaultValue}) {
    double? value = sharedPreferences.getDouble(key);
    return value ?? defaultValue;
  }

  static Future<bool> putStringList(String key, List<String>? value) =>
      value == null ? remove(key) : sharedPreferences.setStringList(key, value);

  static List<String>? getStringList(String key, {List<String>? defaultValue}) {
    List<String>? value = sharedPreferences.getStringList(key);
    return value ?? defaultValue;
  }

  static Future<bool> remove(String key) {
    return sharedPreferences.remove(key);
  }
}
