import 'package:adeptutils/adeptutils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart' as Intl;

export 'package:json_annotation/json_annotation.dart';

/// Created by Luis on 09/02/19.

class AdeptDateTime implements Comparable<AdeptDateTime> {
  static const String FULL_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSS";
  static const String SIMPLE_DATE_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss";

  static const String FORMATTED_DATE_TIME_FORMAT = 'EEE, d MMM yyyy hh:mm a';
  static const String FORMATTED_DATE_TIME_24h_FORMAT = 'EEE, d MMM yyyy HH:mm';
  static const String FORMATTED_DATE_FORMAT = 'EEE, d MMM yyyy';
  static const String FORMATTED_TIME_FORMAT = 'hh:mm a';
  static const String FORMATTED_TIME_FORMAT_24h = 'HH:mm';

  static const String FORMATTED_DATE_TIME_FORMAT_SHORT = 'd/MM/y h:mm a';
  static const String FORMATTED_DATE_TIME_FORMAT_24h_SHORT = 'd/MM/y H:mm';
  static const String FORMATTED_DATE_FORMAT_SHORT = 'd/MM/yy';
  static const String FORMATTED_TIME_FORMAT_SHORT = 'h:mm a';
  static const String FORMATTED_TIME_FORMAT_24h_SHORT = 'H:mm';

  @JsonKey(includeToJson: false, includeFromJson: false)
  int? dateTimeMillis;
  @JsonKey(includeToJson: false, includeFromJson: false)
  DateTime? dateAsDateTime;
  @JsonKey(includeToJson: false, includeFromJson: false)
  DateTime? dateAsDateTimeLocal;
  @JsonKey(includeToJson: false, includeFromJson: false)
  String dateFormat;
  @JsonKey(includeToJson: false, includeFromJson: false)
  String? formattedDateTimeFormat = FORMATTED_DATE_TIME_FORMAT;
  @JsonKey(includeToJson: false, includeFromJson: false)
  String? formattedDateFormat = FORMATTED_DATE_FORMAT;
  @JsonKey(includeToJson: false, includeFromJson: false)
  String? formattedTimeFormat = FORMATTED_TIME_FORMAT;
  @JsonKey(includeToJson: false, includeFromJson: false)
  bool use24h;
  String? date;

  AdeptDateTime({
    this.date,
    int? dateTimeMillis,
    DateTime? dateAsDateTime,
    bool? use24h,
    String? dateFormatStandard,
    this.formattedDateTimeFormat,
    this.formattedDateFormat,
    this.formattedTimeFormat,
    BuildContext? context,
  })  : use24h = use24h ??
            FlutterViewUtils.get(context: context)
                .platformDispatcher
                .alwaysUse24HourFormat,
        dateFormat = dateFormatStandard ?? FULL_DATE_TIME_FORMAT {
    if (date == null) {
      if (dateAsDateTime != null) setDateAsDateTime(dateAsDateTime);
      if (dateTimeMillis != null) setDateTimeMillis(dateTimeMillis);
    }
  }

  String format({String? format, Locale? locale, BuildContext? context}) {
    try {
      locale ??=
          FlutterViewUtils.get(context: context).platformDispatcher.locale;
      return Intl.DateFormat(
              format ?? getFormattedDateTimeFormat(), locale.toString())
          .format(getDateAsDateTimeLocal()!);
    } catch (e) {
      print(e);
      try {
        return Intl.DateFormat(format ?? getFormattedDateTimeFormat())
            .format(getDateAsDateTimeLocal()!);
      } catch (e) {
        print(e);
        return Intl.DateFormat().format(getDateAsDateTimeLocal()!);
      }
    }
  }

  String formatted() => format(format: getFormattedDateTimeFormat());
  String formattedShort() {
//    if(formattedDateTimeFormat != null) return format(format: formattedDateTimeFormat);
    return format(
        format:
            '$FORMATTED_DATE_TIME_FORMAT_SHORT ${use24h ? FORMATTED_TIME_FORMAT_24h_SHORT : FORMATTED_TIME_FORMAT_SHORT}');
  }

  String formattedDate() => format(format: getFormattedDateFormat());
  String formattedDateShort() => format(format: FORMATTED_DATE_FORMAT_SHORT);

  String formattedTime() => format(format: getFormattedTimeFormat());
  String formattedTimeShort() {
//    if(formattedTimeFormat != null) return format(format: formattedTimeFormat);
    return format(
        format: use24h
            ? FORMATTED_TIME_FORMAT_24h_SHORT
            : FORMATTED_TIME_FORMAT_SHORT);
  }

  int getDateTimeMillis() =>
      dateTimeMillis ??
      (dateTimeMillis = getDateAsDateTime()!.millisecondsSinceEpoch);
  void setDateTimeMillis(int value) {
    dateTimeMillis ??= value;
    dateAsDateTime = DateTime.fromMillisecondsSinceEpoch(dateTimeMillis!);
  }

  DateTime? getDateAsDateTime() {
    if (dateAsDateTime == null) {
      if (date != null) {
        try {
          dateAsDateTime = Intl.DateFormat(dateFormat).parse(date!);
        } catch (e) {
          print(e);
          dateAsDateTime = DateTime.tryParse(date!);
        }
      }
      if (dateAsDateTime == null) setDateAsDateTime(DateTime.now());
    }
    if (date == null) setDateAsDateTime(dateAsDateTime);
    return dateAsDateTime;
  }

  setDateAsDateTime(DateTime? value) {
    dateAsDateTime ??= value;
    if (dateAsDateTime != null) {
      dateAsDateTimeLocal = DateTime.fromMillisecondsSinceEpoch(
          dateAsDateTime!.millisecondsSinceEpoch);
      date = format(format: dateFormat);
    }
  }

  DateTime? getDateAsDateTimeLocal() =>
      dateAsDateTimeLocal ?? (dateAsDateTimeLocal = getDateAsDateTime());
  setDateAsDateTimeLocal(DateTime value) => dateAsDateTimeLocal = value;

  String getDateFormat() => dateFormat;
  setDateFormat(String value) => dateFormat = value;

  String getFormattedDateTimeFormat() =>
      formattedDateTimeFormat ??
      '${getFormattedDateFormat()} ${getFormattedTimeFormat()}';
  setFormattedDateTimeFormat(String value) => formattedDateTimeFormat = value;

  String getFormattedDateFormat() =>
      formattedDateFormat ?? FORMATTED_DATE_FORMAT;
  setFormattedDateFormat(String value) => formattedDateFormat = value;

  String getFormattedTimeFormat() =>
      formattedTimeFormat ??
      (use24h ? FORMATTED_TIME_FORMAT_24h : FORMATTED_TIME_FORMAT);
  setFormattedTimeFormat(String value) => formattedTimeFormat = value;

  String? getDate() {
    if (date == null) getDateAsDateTime();

    ///this generates a non null date
    return date;
  }

  setDate(String value) => date ??= value;

  @override
  int get hashCode => getDate().hashCode;

  @override
  bool operator ==(other) {
    if (other is AdeptDateTime) return getDate() == other.getDate();
    return false;
  }

  bool operator >(other) {
    if (other is! AdeptDateTime) return false;
    return getDateTimeMillis() > other.getDateTimeMillis();
  }

  bool operator >=(other) {
    if (other is! AdeptDateTime) return false;
    return getDateTimeMillis() >= other.getDateTimeMillis();
  }

  bool operator <(other) {
    if (other is! AdeptDateTime) return false;
    return getDateTimeMillis() < other.getDateTimeMillis();
  }

  bool operator <=(other) {
    if (other is! AdeptDateTime) return false;
    return getDateTimeMillis() <= other.getDateTimeMillis();
  }

  @override
  int compareTo(AdeptDateTime other) {
    if (getDateAsDateTime() == null && other.getDateAsDateTime() == null) {
      return 0;
    }
    if (getDateAsDateTime() != null && other.getDateAsDateTime() == null) {
      return 1;
    }
    if (getDateAsDateTime() == null && other.getDateAsDateTime() != null) {
      return -1;
    }
    return getDateAsDateTime()!.compareTo(other.getDateAsDateTime()!);
  }

  String? toJson() => getDate();
  Map<String, dynamic> toJsonMap() => <String, dynamic>{'date': getDate()};
  static AdeptDateTime? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? AdeptDateTime(date: json['date']) : null;
  factory AdeptDateTime.fromJson(String? json) => AdeptDateTime(date: json);
}
