import 'package:adeptutils/adeptutils.dart';

class Coordinates extends Jsonable<Coordinates> {
  @JsonKey(name: 'latitude')
  double latitude;
  @JsonKey(name: 'longitude')
  double longitude;

  Coordinates({double? latitude, double? longitude})
      : latitude = latitude ?? 0,
        longitude = longitude ?? 0;

  Location get asLocation => Location(latitude: latitude, longitude: longitude);

  bool get isNotEmpty => !isEmpty;

  bool get isEmpty => (longitude == 0 && latitude == 0);

  bool isDifferentThan(Coordinates? location) =>
      location == null ||
      (latitude != location.latitude && longitude != location.longitude);

  ///
  /// Using Haversine formula
  ///
  double distanceTo(
      {double? latitude, double? longitude, Coordinates? coordinates}) {
    assert((latitude != null && longitude != null) || coordinates != null);
    if (coordinates != null) {
      return distanceTo(
          latitude: coordinates.latitude, longitude: coordinates.longitude);
    }
    return Location.distanceBetween(
        this.latitude, this.longitude, latitude!, longitude!);
  }

  ///
  /// Using Euclidean formula
  ///
  double fastDistanceTo(
      {double? latitude, double? longitude, Coordinates? coordinates}) {
    assert((latitude != null && longitude != null) || coordinates != null);
    if (coordinates != null) {
      return fastDistanceTo(
          latitude: coordinates.latitude, longitude: coordinates.longitude);
    }
    return Location.fastDistanceBetween(
        this.latitude, this.longitude, latitude!, longitude!);
  }

  @override
  Map<String, dynamic> toJson() => {
        'longitude': longitude,
        'latitude': latitude,
      };
  @override
  Coordinates? fromJsonMap(Map<String, dynamic>? json) =>
      json == null ? null : Coordinates.fromJson(json);

  factory Coordinates.fromJson(Map<String, dynamic> json) => Coordinates(
        longitude: (json['longitude'] as num?)?.toDouble(),
        latitude: (json['latitude'] as num?)?.toDouble(),
      );
  factory Coordinates.fromLocation(Location location) =>
      Coordinates(latitude: location.latitude, longitude: location.longitude);
}
